@@ -27,6 +27,13 @@ module Yesod.Core
     , logWarn
     , logError
     , logOther
+      -- * Sessions
+    , SessionBackend (..)
+    , defaultClientSessionBackend
+    , clientSessionBackend
+    , loadClientSession
+    , Header(..)
+    , BackendSession
     -- * JS loaders
     , loadJsYepnope
     , ScriptLoadPosition (..)
@@ -44,6 +51,7 @@ module Yesod.Core
     ) where
 
 import Yesod.Internal.Core
+import Yesod.Internal (Header(..))
 import Yesod.Content
 import Yesod.Dispatch
 import Yesod.Handler
