@@ -76,6 +76,7 @@ module Yesod.Handler
     , setSession
     , setSessionBS
     , deleteSession
+    , clearSession
       -- ** Ultimate destination
     , setUltDest
     , setUltDestCurrent
@@ -137,7 +138,11 @@ import qualified Network.Wai as W
 import qualified Network.HTTP.Types as H
 
 import Text.Hamlet
-import qualified Text.Blaze.Renderer.Text
+#if MIN_VERSION_blaze_html(0, 5, 0)
+import qualified Text.Blaze.Html.Renderer.Text as RenderText
+#else
+import qualified Text.Blaze.Renderer.Text as RenderText
+#endif
 import qualified Data.Text as T
 import Data.Text.Encoding (encodeUtf8, decodeUtf8With)
 import Data.Text.Encoding.Error (lenientDecode)
@@ -160,15 +165,20 @@ import Blaze.ByteString.Builder (toByteString)
 import Data.Text (Text)
 import Yesod.Message (RenderMessage (..))
 
+#if MIN_VERSION_blaze_html(0, 5, 0)
+import Text.Blaze.Html (toHtml, preEscapedToMarkup)
+#define preEscapedText preEscapedToMarkup
+#else
 import Text.Blaze (toHtml, preEscapedText)
+#endif
 
 import qualified Yesod.Internal.Cache as Cache
 import Yesod.Internal.Cache (mkCacheKey, CacheKey)
 import Data.Typeable (Typeable)
 import qualified Data.IORef as I
-import Control.Monad.Trans.Resource
 import Control.Exception.Lifted (catch)
 import Control.Monad.Trans.Control
+import Control.Monad.Trans.Resource
 import Control.Monad.Base
 import Yesod.Routes.Class
 
@@ -527,7 +537,7 @@ msgKey = "_MSG"
 --
 -- See 'getMessage'.
 setMessage :: Html -> GHandler sub master ()
-setMessage = setSession msgKey . T.concat . TL.toChunks . Text.Blaze.Renderer.Text.renderHtml
+setMessage = setSession msgKey . T.concat . TL.toChunks . RenderText.renderHtml
 
 -- | Sets a message in the user's session.
 --
@@ -700,6 +710,12 @@ setSessionBS k = modify . modSession . Map.insert k
 deleteSession :: Text -> GHandler sub master ()
 deleteSession = modify . modSession . Map.delete
 
+-- | Clear all session variables.
+--
+-- Since: 1.0.1
+clearSession :: GHandler sub master ()
+clearSession = modify $ \x -> x { ghsSession = Map.empty }
+
 modSession :: (SessionMap -> SessionMap) -> GHState -> GHState
 modSession f x = x { ghsSession = f $ ghsSession x }
 
@@ -772,14 +788,9 @@ handlerToYAR y s toMasterRoute render errorHandler rr murl sessionMap h =
     types = httpAccept $ reqWaiRequest rr
     errorHandler' = localNoCurrent . errorHandler
 
-type HeaderRenderer = [Header]
-                   -> ContentType
-                   -> SessionMap
-                   -> [(CI H.Ascii, H.Ascii)]
-
-yarToResponse :: HeaderRenderer -> YesodAppResult -> W.Response
-yarToResponse _ (YARWai a) = a
-yarToResponse renderHeaders (YARPlain s hs ct c sessionFinal) =
+yarToResponse :: YesodAppResult -> [(CI H.Ascii, H.Ascii)] -> W.Response
+yarToResponse (YARWai a) _ = a
+yarToResponse (YARPlain s hs _ c _) extraHeaders =
     case c of
         ContentBuilder b mlen ->
             let hs' = maybe finalHeaders finalHeaders' mlen
@@ -787,11 +798,10 @@ yarToResponse renderHeaders (YARPlain s hs ct c sessionFinal) =
         ContentFile fp p -> W.ResponseFile s finalHeaders fp p
         ContentSource body -> W.ResponseSource s finalHeaders body
   where
-    finalHeaders = renderHeaders hs ct sessionFinal
+    finalHeaders = extraHeaders ++ map headerToPair hs
     finalHeaders' len = ("Content-Length", S8.pack $ show len)
                       : finalHeaders
 
-
 httpAccept :: W.Request -> [ContentType]
 httpAccept = parseHttpAccept
            . fromMaybe mempty
@@ -831,13 +841,8 @@ newIdent = do
 redirectToPost :: RedirectUrl master url => url -> GHandler sub master a
 redirectToPost url = do
     urlText <- toTextUrl url
-    hamletToRepHtml
-#if GHC7
-            [hamlet|
-#else
-            [$hamlet|
-#endif
-\<!DOCTYPE html>
+    hamletToRepHtml [hamlet|
+$doctype 5
 
 <html>
     <head>
@@ -922,12 +927,12 @@ instance MonadBaseControl IO (GHandler sub master) where
             f $ liftM StH . runInBase . (\(GHandler r) -> r reader)
     restoreM (StH base) = GHandler $ const $ restoreM base
 
-instance Resource (GHandler sub master) where
-    type Base (GHandler sub master) = IO
-    resourceLiftBase = liftIO
-    resourceBracket_ a b c = control $ \run -> resourceBracket_ a b (run c)
-instance ResourceUnsafeIO (GHandler sub master) where
-    unsafeFromIO = liftIO
-instance ResourceThrow (GHandler sub master) where
-    resourceThrow = liftIO . throwIO
-instance ResourceIO (GHandler sub master)
+instance MonadUnsafeIO (GHandler sub master) where
+    unsafeLiftIO = liftIO
+instance MonadThrow (GHandler sub master) where
+    monadThrow = liftIO . throwIO
+instance MonadResource (GHandler sub master) where
+    allocate a = lift . allocate a
+    register = lift . register
+    release = lift . release
+    resourceMask = lift . resourceMask
