@@ -24,8 +24,7 @@ module Yesod.Internal
     , runUniqueList
     , toUnique
       -- * Names
-    , sessionName
-    , nonceKey
+    , tokenKey
     ) where
 
 import Text.Hamlet (HtmlUrl, hamlet, Html)
@@ -45,12 +44,6 @@ import Data.Text.Lazy.Builder (Builder)
 import Network.HTTP.Types (Ascii)
 import Web.Cookie (SetCookie (..))
 
-#if GHC7
-#define HAMLET hamlet
-#else
-#define HAMLET $hamlet
-#endif
-
 -- | Responses to indicate some form of an error occurred. These are different
 -- from 'SpecialResponse' in that they allow for custom error pages.
 data ErrorResponse =
@@ -76,9 +69,9 @@ langKey = "_LANG"
 data Location url = Local url | Remote Text
     deriving (Show, Eq)
 locationToHtmlUrl :: Location url -> HtmlUrl url
-locationToHtmlUrl (Local url) = [HAMLET|\@{url}
+locationToHtmlUrl (Local url) = [hamlet|\@{url}
 |]
-locationToHtmlUrl (Remote s) = [HAMLET|\#{s}
+locationToHtmlUrl (Remote s) = [hamlet|\#{s}
 |]
 
 newtype UniqueList x = UniqueList ([x] -> [x])
@@ -101,11 +94,8 @@ newtype Head url = Head (HtmlUrl url)
 newtype Body url = Body (HtmlUrl url)
     deriving Monoid
 
-nonceKey :: IsString a => a
-nonceKey = "_NONCE"
-
-sessionName :: IsString a => a
-sessionName = "_SESSION"
+tokenKey :: IsString a => a
+tokenKey = "_TOKEN"
 
 type CssBuilderUrl a = (a -> [(Text, Text)] -> Text) -> Builder
 
