@@ -36,23 +36,25 @@ data Request = Request
     , reqWaiRequest :: W.Request
       -- | Languages which the client supports.
     , reqLangs :: [Text]
-      -- | A random, session-specific nonce used to prevent CSRF attacks.
-    , reqNonce :: Maybe Text
+      -- | A random, session-specific token used to prevent CSRF attacks.
+    , reqToken :: Maybe Text
     }
 
 parseWaiRequest :: W.Request
                 -> [(Text, ByteString)] -- ^ session
-                -> Maybe a
+                -> Bool
                 -> IO Request
-parseWaiRequest env session' key' = parseWaiRequest' env session' key' <$> newStdGen
+parseWaiRequest env session' useToken =
+    parseWaiRequest' env session' useToken <$> newStdGen
 
 parseWaiRequest' :: RandomGen g
                  => W.Request
                  -> [(Text, ByteString)] -- ^ session
-                 -> Maybe a
+                 -> Bool
                  -> g
                  -> Request
-parseWaiRequest' env session' key' gen = Request gets'' cookies' env langs'' nonce
+parseWaiRequest' env session' useToken gen = 
+    Request gets'' cookies' env langs'' token
   where
     gets' = queryToQueryText $ W.queryString env
     gets'' = map (second $ fromMaybe "") gets'
@@ -73,14 +75,16 @@ parseWaiRequest' env session' key' gen = Request gets'' cookies' env langs'' non
     -- language in the list.
     langs'' = addTwoLetters (id, Set.empty) langs'
 
-    -- If sessions are disabled nonces should not be used (any
-    -- nonceKey present in the session is ignored). If sessions
-    -- are enabled and a session has no nonceKey a new one is
+    -- If sessions are disabled tokens should not be used (any
+    -- tokenKey present in the session is ignored). If sessions
+    -- are enabled and a session has no tokenKey a new one is
     -- generated.
-    nonce = case (key', lookup nonceKey session') of
-                (Nothing, _) -> Nothing
-                (_, Just x)  -> Just $ decodeUtf8With lenientDecode x
-                _            -> Just $ pack $ randomString 10 gen
+    token = if not useToken
+              then Nothing
+              else Just $ maybe
+                            (pack $ randomString 10 gen)
+                            (decodeUtf8With lenientDecode)
+                            (lookup tokenKey session')
 
 addTwoLetters :: ([Text] -> [Text], Set.Set Text) -> [Text] -> [Text]
 addTwoLetters (toAdd, exist) [] =
