@@ -35,7 +35,7 @@ getRootR = do
     Nothing <- cacheLookup key
     return ()
 
-cacheTest :: [Spec]
+cacheTest :: Spec
 cacheTest =
   describe "Test.Cache"
     [ it "works" works
