@@ -25,7 +25,7 @@ instance Yesod App
 
 getHomeR :: Handler RepHtml
 getHomeR = defaultLayout $ toWidget [hamlet|
-!!!
+$doctype 5
 
 <html>
   <body>
@@ -52,7 +52,7 @@ postAfterRunRequestBodyR = do
    _ <- error $ show x
    getHomeR
 
-errorHandlingTest :: [Spec]
+errorHandlingTest :: Spec
 errorHandlingTest = describe "Test.ErrorHandling"
     [ it "says not found" caseNotFound
     , it "says 'There was an error' before runRequestBody" caseBefore
