@@ -30,7 +30,7 @@ getRedirR = do
     setHeader "foo" "bar"
     redirectWith status301 RootR
 
-exceptionsTest :: [Spec]
+exceptionsTest :: Spec
 exceptionsTest = describe "Test.Exceptions"
     [ it "500" case500
     , it "redirect keeps headers" caseRedirect
