@@ -10,7 +10,7 @@ import Yesod.Internal.TestApi (randomString, parseWaiRequest')
 import Yesod.Request (Request (..))
 import Test.Hspec
 
-randomStringSpecs :: [Spec]
+randomStringSpecs :: Spec
 randomStringSpecs = describe "Yesod.Internal.Request.randomString"
   [ it "looks reasonably random" looksRandom
   , it "does not repeat itself" $ noRepeat 10 100
@@ -27,35 +27,35 @@ noRepeat len n = length (nub $ map (randomString len . mkStdGen) [1..n]) == n
 
 -- For convenience instead of "(undefined :: StdGen)".
 g :: StdGen
-g = undefined
+g = error "test/YesodCoreTest/InternalRequest.g"
 
 
-nonceSpecs :: [Spec]
-nonceSpecs = describe "Yesod.Internal.Request.parseWaiRequest (reqNonce)"
-  [ it "is Nothing if sessions are disabled" noDisabledNonce
-  , it "ignores pre-existing nonce if sessions are disabled" ignoreDisabledNonce
-  , it "uses preexisting nonce in session" useOldNonce
-  , it "generates a new nonce for sessions without nonce" generateNonce
+tokenSpecs :: Spec
+tokenSpecs = describe "Yesod.Internal.Request.parseWaiRequest (reqToken)"
+  [ it "is Nothing if sessions are disabled" noDisabledToken
+  , it "ignores pre-existing token if sessions are disabled" ignoreDisabledToken
+  , it "uses preexisting token in session" useOldToken
+  , it "generates a new token for sessions without token" generateToken
   ]
 
-noDisabledNonce :: Bool
-noDisabledNonce = reqNonce r == Nothing where
-  r = parseWaiRequest' defaultRequest [] Nothing g
+noDisabledToken :: Bool
+noDisabledToken = reqToken r == Nothing where
+  r = parseWaiRequest' defaultRequest [] False g
 
-ignoreDisabledNonce :: Bool
-ignoreDisabledNonce = reqNonce r == Nothing where
-  r = parseWaiRequest' defaultRequest [("_NONCE", "old")] Nothing g
+ignoreDisabledToken :: Bool
+ignoreDisabledToken = reqToken r == Nothing where
+  r = parseWaiRequest' defaultRequest [("_TOKEN", "old")] False g
 
-useOldNonce :: Bool
-useOldNonce = reqNonce r == Just "old" where
-  r = parseWaiRequest' defaultRequest [("_NONCE", "old")] (Just undefined) g
+useOldToken :: Bool
+useOldToken = reqToken r == Just "old" where
+  r = parseWaiRequest' defaultRequest [("_TOKEN", "old")] True g
 
-generateNonce :: Bool
-generateNonce = reqNonce r /= Nothing where
-  r = parseWaiRequest' defaultRequest [("_NONCE", "old")] (Just undefined) g
+generateToken :: Bool
+generateToken = reqToken r /= Nothing where
+  r = parseWaiRequest' defaultRequest [("_TOKEN", "old")] True g
 
 
-langSpecs :: [Spec]
+langSpecs :: Spec
 langSpecs = describe "Yesod.Internal.Request.parseWaiRequest (reqLangs)"
   [ it "respects Accept-Language" respectAcceptLangs
   , it "respects sessions" respectSessionLang
@@ -67,21 +67,21 @@ langSpecs = describe "Yesod.Internal.Request.parseWaiRequest (reqLangs)"
 respectAcceptLangs :: Bool
 respectAcceptLangs = reqLangs r == ["en-US", "es", "en"] where
   r = parseWaiRequest' defaultRequest
-        { requestHeaders = [("Accept-Language", "en-US, es")] } [] Nothing g
+        { requestHeaders = [("Accept-Language", "en-US, es")] } [] False g
 
 respectSessionLang :: Bool
 respectSessionLang = reqLangs r == ["en"] where
-  r = parseWaiRequest' defaultRequest [("_LANG", "en")] Nothing g
+  r = parseWaiRequest' defaultRequest [("_LANG", "en")] False g
 
 respectCookieLang :: Bool
 respectCookieLang = reqLangs r == ["en"] where
   r = parseWaiRequest' defaultRequest
         { requestHeaders = [("Cookie", "_LANG=en")]
-        } [] Nothing g
+        } [] False g
 
 respectQueryLang :: Bool
 respectQueryLang = reqLangs r == ["en-US", "en"] where
-  r = parseWaiRequest' defaultRequest { queryString = [("_LANG", Just "en-US")] } [] Nothing g
+  r = parseWaiRequest' defaultRequest { queryString = [("_LANG", Just "en-US")] } [] False g
 
 prioritizeLangs :: Bool
 prioritizeLangs = reqLangs r == ["en-QUERY", "en-COOKIE", "en-SESSION", "en", "es"] where
@@ -90,11 +90,12 @@ prioritizeLangs = reqLangs r == ["en-QUERY", "en-COOKIE", "en-SESSION", "en", "e
                            , ("Cookie", "_LANG=en-COOKIE")
                            ]
         , queryString = [("_LANG", Just "en-QUERY")]
-        } [("_LANG", "en-SESSION")] Nothing g
+        } [("_LANG", "en-SESSION")] False g
 
 
-internalRequestTest :: [Spec]
-internalRequestTest = descriptions [ randomStringSpecs
-                                   , nonceSpecs
-                                   , langSpecs
-                                   ]
+internalRequestTest :: Spec
+internalRequestTest = describe "Test.InternalRequestTest" 
+    [ randomStringSpecs
+    , tokenSpecs
+    , langSpecs
+    ]
