@@ -18,9 +18,9 @@ mkYesod "Y" [parseRoutes|
 instance Yesod Y
 
 getRootR :: Handler RepHtml
-getRootR = defaultLayout $ addHamlet [hamlet|<a href=@{RootR}>|]
+getRootR = defaultLayout $ toWidget [hamlet|<a href=@{RootR}>|]
 
-linksTest :: [Spec]
+linksTest :: Spec
 linksTest = describe "Test.Links"
     [ it "linkToHome" case_linkToHome
     ]
